import os
from dotenv import load_dotenv
import functools
import asyncio
import discord
from discord.ext import commands
from werewolf_game import Game as Werewolf_Game


load_dotenv()
TOKEN = os.getenv('DISCORD_TOKEN')
if TOKEN is None:
	print("Missing discord token!")
	exit(1)

PREFIX = '$w '
bot = commands.Bot(command_prefix=commands.when_mentioned_or(PREFIX))


@bot.event
async def on_ready():
	await bot.change_presence(status=discord.Status.idle, activity=discord.Game('One Night Ultimate Werewolf'))
	print('We have logged in as {0.user}'.format(bot))


# game commands

game_instances = {}


@bot.group(help="werewolf game", description="Prefix for the One Night Ultimate Werewolf commands.")
async def game(ctx):
	if ctx.invoked_subcommand is None:
		await ctx.send('Invalid sub command passed...')


@game.command(help="start werewolf game", description="Start One Night Ultimate Werewolf game.")
async def setup(ctx):
	if ctx.channel in game_instances:
		await ctx.send("Game already setup in this channel")
	else:
		game_instances[ctx.channel] = Werewolf_Game(bot, ctx.channel)


def game_running(command):
	@functools.wraps(command)
	async def wrapper(ctx):
		if ctx.channel not in game_instances:
			await ctx.send(f"No game setup yet. Use {PREFIX}game setup")
		elif game_instances[ctx.channel].running:
			await ctx.send("Sorry! A game is already running")
		else:
			await command(ctx)
	return wrapper


@game.command(help="start a round", description="Play one round of One Night Ultimate Werewolf")
@game_running
async def start(ctx):
	await game_instances[ctx.channel].game()


@game.command(help="set players", description="Set the players for the next round.")
@game_running
async def players(ctx):
	await game_instances[ctx.channel].set_players(ctx.message)


@game.command(help="set roles", description="Set the roles for the next round.")
@game_running
async def roles(ctx):
	try:
		await game_instances[ctx.channel].set_roles(ctx.message.content.split()[3:])
	except ValueError as error:
		await ctx.send(error)

# ONLY FOR TESTING
@game.command()
@game_running
async def vote(ctx):
	await game_instances[ctx.channel].vote()


# smaller commands


@bot.command(help="greets you", description="This just says hello back to the message author.")
async def hello(ctx):
	await ctx.send(f"Hello {ctx.message.author.name} :wave:")


@bot.command(help="test bot responsiveness", description="This is a debug function to see if the bot listens to a command.")
async def ping(ctx):
	print("pong")
	await ctx.send("pong")


# developer commands


def developer(command):
	@functools.wraps(command)
	async def wrapper(ctx):
		DEV_ID = 461892912821698562
		if ctx.author.id == DEV_ID:
			await command(ctx)
		else:
			await ctx.send("This command is not for you!")
	return wrapper


@bot.command(help="not for you", description="Shut down the bot.")
@developer
async def logout(ctx):
	await bot.logout()


@bot.command(help="debug")
@developer
async def debug(ctx):
	print("DEBUG")
	print(ctx.message.author.id)

bot.run(TOKEN)
